@@ -1,8 +1,3 @@
-# Copyright (c) 2010 CentralNic Ltd. All rights reserved. This program is
-# free software; you can redistribute it and/or modify it under the same
-# terms as Perl itself.
-# 
-# $Id: Client.pm,v 1.17 2008/01/23 12:23:16 gavin Exp $
 package Net::EPP::Client;
 use bytes;
 use Net::EPP::Protocol;
@@ -10,7 +5,6 @@ use Carp;
 use IO::Socket;
 use IO::Socket::SSL;
 use vars qw($XMLDOM $EPPFRAME);
-use UNIVERSAL qw(isa);
 use strict;
 use warnings;
 
@@ -145,7 +139,6 @@ sub new {
 			croak("Frames requested but Net::EPP::Frame isn't available");
 
 		} else {
-			$self->{'parser'} = XML::LibXML->new;
 			$self->{'class'} = 'Net::EPP::Frame';
 
 		}
@@ -155,15 +148,10 @@ sub new {
 			croak("DOM requested but XML::LibXML isn't available");
 
 		} else {
-			$self->{'parser'} = XML::LibXML->new;
 			$self->{'class'} = 'XML::LibXML::Document';
 
 		}
 
-	} else {
-		# for well-formedness checking:
-		$self->{'parser'} = XML::LibXML->new;
-
 	}
 
 	return bless($self, $package);
@@ -185,9 +173,11 @@ certificate information). See the relevant manpage for examples.
 This method will C<croak()> if connection fails, so be sure to use C<eval()> if
 you want to catch the error.
 
-The return value for C<connect()> will be the EPP E<lt>greetingE<gt> frame
-returned by the server. Please note that the same caveat about blocking applies
-to this method as to C<get_frame()> (see below).
+By default, the return value for C<connect()> will be the EPP E<lt>greetingE<gt>
+frame returned by the server. Please note that the same caveat about blocking
+applies to this method as to C<get_frame()> (see below).
+
+If you want to get the greeting yourself, set C<$params{no_greeting}>.
 
 =cut
 
@@ -202,7 +192,7 @@ sub connect {
 
 	}
 
-	return $self->get_frame;
+	return ($params{'no_greeting'} ? 1 : $self->get_frame);
 
 }
 
@@ -308,7 +298,7 @@ sub get_return_value {
 
 	} else {
 		my $document;
-		eval { $document = $self->{'parser'}->parse_string($xml) };
+		eval { $document = $self->parser->parse_string($xml) };
 		if (!defined($document) || $@ ne '') {
 			chomp($@);
 			croak(sprintf("Frame from server wasn't well formed: %s\n\nThe XML looks like this:\n\n%s\n\n", $@, $xml));
@@ -373,7 +363,7 @@ sub send_frame {
 	}
 
 	if ($wfcheck == 1) {
-		eval { $self->{'parser'}->parse_string($xml) };
+		eval { $self->parser->parse_string($xml) };
 		if ($@ ne '') {
 			chomp($@);
 			croak(sprintf("Frame from server wasn't well formed: %s\n\nThe XML looks like this:\n\n%s\n\n", $@, $xml));
@@ -398,35 +388,14 @@ connection out of sync with the server.
 
 sub disconnect {
 	my $self = shift;
-	$self->{'connection'}->close;
+	$self->{'connection'}->close if ($self->{'connection'});
 	return 1;
 }
 
-=pod
-
-=head1 AUTHOR
-
-CentralNic Ltd (L<http://www.centralnic.com/>).
-
-=head1 COPYRIGHT
-
-This module is (c) 2006 CentralNic Ltd. This module is free software; you can
-redistribute it and/or modify it under the same terms as Perl itself.
-
-=head1 SEE ALSO
-
-=over
-
-=item * L<Net::EPP::Frame>
-
-=item * L<Net::EPP::Proxy>
-
-=item * RFCs 4930 and RFC 4934, available from L<http://www.ietf.org/>.
-
-=item * The CentralNic EPP site at L<http://www.centralnic.com/resellers/epp>.
-
-=back
-
-=cut
+sub parser {
+	my $self = shift;
+	$self->{'parser'} = XML::LibXML->new if (!$self->{'parser'});
+	return $self->{'parser'};
+}
 
 1;
